<?php

/**
 * Returns parameters to configure plug-in.
 * Mandatory.
 *
 * @return array
 */
function Example_GetConfig()
{
    return array(
        'friendlyName' => pa_localized_string('plugin_title'),          # title  - A plugin title that will be shown in plugin configuration page and in the gateway selection options.
                                                                        #          Note: localization is available here.
        'connection' => array(                                          # "connection" is a section name
            'type' => 'section',                                        # type => section means that parameters from 'items' will be grouped in the section with the title of 'friendly' value
            'friendlyName' => pa_localized_string('conn_setting'),      # friendlyName - A display name that will be shown in plugin config
            'items' => array(                                           # items - Section items
                'gateway_url' => array(                                 # Item called by 'gateway_url'
                    'type' => 'text',                                   # Parameter's type is text
                    'friendlyName' => pa_localized_string('gtw_url'),   # Display text for 'gateway_url' parameter
                    'default' => 'https://demo-payment-gateway.com',    # Parameter's default value
                    'required' => true                                  # Parameter is required and will be marked with red symbol '*' in the plugin configuration page
                ),
                'auth_code' => array(                                   # Item called by 'auth_code'
                    'type' => 'integer',                                # 'auth_code' parameter's type is integer
                    'friendlyName' => pa_localized_string('auth_code'), # Display text for 'auth_code' parameter
                    'min' => 100,                                       # Minimum of parameter value
                    'max' => 999                                        # Maximum of parameter value
                )
            )
        ),
        'format' => array(                                              # Protocol format (it is just an example :))
            'type' => 'dropdown',                                       # Type is dropdown. Shown as combobox
            'options' => array(                                         # Parameter's options
                'xml', 'json', 'csv'
            ),
            'default' => 'json',                                        # Parameter's default value
            'friendlyName' => pa_localized_string('protocol_format')    # Display text
        ),
        'test_mode' => array(                                           # Test mode parameter (without a section in this example)
            'type' => 'yesno',                                          # Can be 'yes' or 'no'. "Test mode" will be shown as checkbox
            'friendlyName' => pa_localized_string('test_mode'),         # Display name for the item
            'default' => 'yes'                                          # Default value is 'yes'
        )
    );
}

/**
 * Returns a list of currencies supported by gateway.
 * Optional
 *
 * @return array of currencies
 */
function Example_GetSupportedCurrencies()
{
    $return = array(
        'EUR', # Euro
        'BGN', # Bulgarian Lev
        'HRK', # Croatian Kuna
        'CZK', # Czech Koruna
        'DKK', # Danish Krone
        'GIP', # Gibraltar Pound
        'GBP', # Pounds
        'HUF', # Hungarian Forint
        'ISK', # Iceland Króna
        'CHF', # Swiss Franc
        'NOK', # Norwegian Krone
        'PLN', # Poland Złoty
        'RON', # Romanian Leu
        'SEK', # Swedish Krona
    );
    return $return;
}

/**
 * Validates plug-in's stored configuration
 * Optional
 *
 * @param $params - array of
 * 			- config
 *			- environment
 *
 * @return array with status "APPROVED" if plug-in's configuration is valid, or "DECLINED" otherwise
 */
function Example_ValidateConfig($params) {

    $gtwUrl = $params['config']['gateway_url'];

    if (preg_match('/^(?:http(s)?:\/\/){1}[\w.-]+(?:\.[\w\.-]+)+[\w\-\._~:\/?#[\]@!\$&\'\(\)\*\+,;=.]+$/', $gtwUrl))
        return array(STATUS => STATUS_APPROVED);

    return array(
        STATUS => STATUS_DECLINED,
        TEXT   => array(
            'customer_message' => pa_localized_string('invalid_url'),
            'vendor_message'   => pa_localized_string('invalid_url')
        )
    );
}

/**
 * Tests gateway availability using plug-in's specified config.
 * Optional
 *
 * @param $params - array of
 * 			- config
 *			- environment
 *
 * @return array with status "APPROVED" if connection test is successful, or "DECLINED" otherwise
 */
function Example_TestConnection($params){
    $config = $params['config'];
    if (!$config['test_mode']) {
        $auth = $config['auth_code'];
    }

    $curl = curl_init();
    curl_setopt($curl, CURLOPT_VERBOSE, true);
    curl_setopt($curl, CURLOPT_FAILONERROR, true);
    curl_setopt($curl, CURLOPT_URL, $config['gateway_url']);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);

    if (isset($auth)) {
        curl_setopt($curl, CURLOPT_HTTPHEADER, array('X-Auth: '.$auth));
    }

    try {
        curl_exec($curl);
    } catch (Exception $e) {
        return array(
            STATUS => STATUS_DECLINED,
            TEXT => array(
                'vendor_message' => $e->getMessage()
            )
        );
    }

    return array(STATUS => STATUS_APPROVED);
}

/**
 * Implements redirection to the payment gateway page.
 * Mandatory if redirection to the gateway site is required
 *
 * @param $params - array of
 * 			- ref_no
 * 			- transaction_amount
 * 			- currency_iso
 * 			- account_info
 * 			- document_info
 * 			- config
 *			- environment
 *
 * @return array with redirect attributes or error description (see description above)
 */
function Example_Redirect( $params ) {
    $config   = $params['config'];
    $ref_no   = $params['ref_no'];
    $env      = $params['environment'];

    $redirect_hash = array(
        'url' => $config['gateway_url'],
        'method' => 'POST',
        'attrs' => array(
            'ref_no' => $ref_no,
            'amount' => $params['transaction_amount'],
            'currency' => $params['currency_iso'],
            'accept_url' => $env['return_url_ok'],
            'decline_url' => $env['return_url_failed'],
            'enable_tokens' => 1
        )
    );

    return array(
        STATUS => STATUS_REDIRECT,
        REDIRECT_HASH => $redirect_hash
    );
}

/**
 * Processes callback data received from a payment gateway.
 * Optional
 *
 *
 * @param $params - array of
 * 			- ref_no
 * 			- transaction_amount
 * 			- currency_iso
 * 			- account_info
 * 			- document_info
 * 			- payment_method
 * 			- previous_transaction_data
 * 			- callback_params
 * 			- config
 *			- environment
 *
 * @return array with Callback transaction result (see description above)
 */
function Example_Callback( $params ) {
    $callback_params = $params['callback_params'];

    $result = $callback_params['returnResult'];
    $txn_id = $callback_params['txn_id'];

    switch ($result) {
        case 'ok' :
            $status = STATUS_APPROVED;
            $desc   = 'Success';

            if (isset($callback_params['token']) && isset($callback_params['public_name'])) {
                $new_method = array(
                    'token' => $callback_params['token'],
                    'paymethod_name' => $callback_params['public_name'],
                    // External ID is used to identify Payment Method
                    'external_id' => md5($callback_params['token']),
                    'initial_status' => PAYMETHOD_STATUS_OK
                );
            }

            if (isset($new_method) && isset($callback_params['expires'])) {
                $new_method['exp_date'] = $callback_params['expires'];
            }

            break;

        case 'fail' :
            $status = STATUS_DECLINED;
            $desc   = 'Declined';
            break;

        default :
            $status = STATUS_ERROR;
            $desc   = "Unknown status '$result'";
            break;
    };

    $details['desc'] = $desc;
    $details['trans_id'] = $txn_id;

    $response = array(
        STATUS              => $status,
        TEXT                => array( 'vendor_message' => $desc ),
        TRANSACTION_DETAILS => $details
    );

    if (isset($new_method)) {
        $response[ADD_NEW_METHOD] = $new_method;
    }

    return $response;
}

/**
 * Implements Sell transaction with a payment gateway.
 * Mandatory for tokenization functionality implementation
 *
 * @param $params - array of
 * 			- ref_no
 * 			- transaction_amount
 * 			- currency_iso
 * 			- account_info
 * 			- document_info
 * 			- payment_method
 * 			- config
 *			- environment
 * @return array with Sell transaction result (see description above)
 * @throws Exception
 */
function Example_Sell( $params ) {
    if (!isset($params['payment_method'])) {
        return Example_Redirect($params);
    }

    return array(
        STATUS => STATUS_APPROVED,
        TRANSACTION_DETAILS => array(
            'trans_id' => (string) time()
        )
    );
}

/**
 * Implements voiding a transaction with a payment gateway.
 * Optional
 *
 * @param $params - array of
 * 			- ref_no
 * 			- transaction_amount
 * 			- currency_iso
 * 			- account_info
 * 			- document_info
 * 			- payment_method
 * 			- previous_transaction_data
 * 			- config
 *			- environment
 *
 * @return array with Void transaction result (see description above)
 * @throws Exception
 */
function Example_Void( $params ) {
    if (!isset($params['previous_transaction_data'])) {
        throw new Exception('No previous transaction data');
    }

    return array(
        STATUS => STATUS_APPROVED,
        TRANSACTION_DETAILS => array(
            'trans_id' => (string) time()
        )
    );
}

?>